#!/bin/sh
# $Id: t-mts-dnsbl-0.sh,v 1.15 2013/10/09 03:41:38 ca Exp $
# Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test DNSBL
# uses smtpcnf.sh.
# ----------------------------------------
#

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1
rm -f ${MTMAP} ${ACCMAP}
# create access map for tests
${MM} -t'	' -F ${ACCMAP} <<EOF
meta1all:127.0.0.2	error:550 5.7.2 found 127.0.0.2 in meta1all
from:ok@b.c	quick:ok
to:ok@y.z	quick:ok
EOF

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar {
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
flags= {access};
wait_for_server=4;
}
EOF

# ----------------------------------------
# reject by dnsbl
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
if ${SHELL} ${SCRIPTS}/smtpcnf.sh -Z 'smtpc2' -c '-E' -E 0 -A 0 -X 0 "$@" -S"s${TEST}@b.c" -R"r${TEST}@y.z"
then
    ERR=false
    ${VERBOSE} && echo "test ${TEST} succeeded"
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${GREP} 'MAIL=error' ${CLTL} >/dev/null
then
  :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# reject by dnsbl, but delay
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar {
Log_Level = 12;
nameserver = ${NS};
DNS_timeout = 5;
dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
flags= {access, delay_checks};
wait_for_server=4;
}
EOF

if ${SHELL} ${SCRIPTS}/smtpcnf.sh -a '-d 8' -Z 'smtpc2' -c '-E' -E 0 -A 0 -X 0 "$@" -S"s${TEST}@b.c" -R"r${TEST}@y.z"
then
    ERR=false
    ${VERBOSE} && echo "test ${TEST} succeeded"
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${GREP} 'RCPT=error' ${CLTL} >/dev/null
then
  :
else
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# reject by dnsbl, but override by sender address
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar {
Log_Level = 19;
nameserver = ${NS};
DNS_timeout = 5;
dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
flags= {access, delay_checks};
wait_for_server=4;
}
EOF

if ${SHELL} ${SCRIPTS}/smtpcnf.sh -a '-d 8' -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"ok@b.c" -R"r${TEST}@y.z"
then
    ERR=false
    ${VERBOSE} && echo "test ${TEST} succeeded"
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${GREP} 'RCPT=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# reject by dnsbl, but override by RCPT address
TEST=4
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar {
Log_Level = 19;
nameserver = ${NS};
DNS_timeout = 5;
dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
flags= {access, delay_checks};
wait_for_server=4;
}
EOF

if ${SHELL} ${SCRIPTS}/smtpcnf.sh -a '-d 8' -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"s${TEST}@b.c" -R"ok@y.z"
then
    ERR=false
    ${VERBOSE} && echo "test ${TEST} succeeded"
else
    echo "$0: test ${TEST} failed"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${GREP} 'RCPT=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# ----------------------------------------
# reject by dnsbl, but override via IP
TEST=5
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then

rm -f ${ACCMAP}
# create access map for tests
${MM} -t'	' -F ${ACCMAP} <<EOF
meta1all:127.0.0.2	error:550 5.7.2 found 127.0.0.2 in meta1all
cltaddr:127.0.0.1	quick:ok
EOF


${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar {
Log_Level = 19;
nameserver = ${NS};
DNS_timeout = 5;
dnsbl { domain = all.meta1.org.; tag = "meta1all"; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
daemon_address = localhost:${SRVPORT};
flags= {access, delay_checks};
wait_for_server=4;
}
EOF

if ${SHELL} ${SCRIPTS}/smtpcnf.sh -a '-d 8' -Z 'smtpc2' -c '-E' -E 1 -A 1 -X 1 "$@" -S"OK@b.c" -R"r${TEST}@y.z"
then
    ERR=false
    ${VERBOSE} && echo "test ${TEST} succeeded"
else
    echo "$0: test ${TEST} failed: $?"
    ERR=true
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
if ${GREP} 'RCPT=error' ${CLTL} >/dev/null
then
  ERR=true
fi
if ${ERR}
then
    echo "$0: test ${TEST} check failed"
    ERRS=`expr ${ERRS} + 1 `
    test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi


# clean up
rm -f ${MTMAP} ${ACCMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
