#!/bin/sh
# $Id: t-mts-occ-0.sh,v 1.3 2013/09/15 00:58:40 ca Exp $
#
# ----------------------------------------
# test outgoing connection restrictions (oc{i,m}).
# fall over to next "MX" (first host doesn't run a sink).
# uses mcpmts.sh
# ----------------------------------------
#

if test "X"${MTA_TIMING} = "X"
then
  echo "SKIPPED: $0: MTA_TIMING is not set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create directory to pass socket
test -d smtps || mkdir smtps

getip2 2

killit()
{
rm -f ${QMGRCONFMAP} ${MTMAP}
exit 1
}

rm -f ${QMGRCONFMAP} ${MTMAP} ${OUT}

# create restrictions: maximum concurrency: 1, 1
${CAT} > ${QMGRCONF} <<EOF
oci:${H2}	2
ocm:${H2}	3
EOF
# no limits on sink for now
# oci:127	2
# ocm:127	3
# create map
${MM} -t'	' -F ${QMGRCONFMAP} < ${QMGRCONF}
trap killit 2

# ----------------------------------------
${CAT} >${SMXCNF} <<EOF
smtps { log_level = 14; pass_fd_socket = smtps/smtpsfd; wait_for_server=4;
	listen_socket { type=inet; port=${SRVPORT}; }
	start_action = pass; user = ${USER};
	path = ../smtps/smtps;
	arguments = "smtps -f ${SMXCNF} ${MTA_SERVER_OPTIONS} -d 3";
}
smtpc { log_level = 14; remote_port = ${SNKPORT}; wait_for_server=4;
	io_timeout=5s;
	start_action = wait; user = ${USER};
	path = "../smtpc/smtpc"; arguments = "smtpc -f ${SMXCNF} -d 3";
}
qmgr { log_level = 14; wait_for_server=4; wait_for_client=4; 
 IBDB { open_TAs_max=32; commit_delay_max=1000;}
	retry_delay_min = 300;
	start_action = wait; user = ${USER};
	# restart_dependencies = { smtps,smtpc,smar, };
	path = "../qmgr/qmgr"; arguments = "qmgr -f ${SMXCNF} -d 6";
}
smar { log_level = 14;
	start_action = wait; user = ${USER};
	# restart_dependencies = { smtps,qmgr };
	path = "../smar/smar"; arguments = "smar -f ${SMXCNF}";
}
EOF

${CAT} > ${MT} <<EOF
dom1.tld	[10.1.1.1]
dom2.tld	[$H2] [127.0.0.1]
EOF

# ----------------------------------------
# 
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
ERR=false
clearlogs

if ${SHELL} ${SD}/mcpmts.sh -Y '-w1' -c '-s 10 -D dom2.tld -F dom1.tld' -N -s 20 -A 10 -E 10 -1
then
  if ${GREP} 'Recipients *10$' ${SRVL} >/dev/null
  then
    if ${GREP} "occ_ip=${H2},.*, cur_conc=0, " qmgr.log >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: wrong cur_conc"
      ERR=true
    fi
  else
    ${VERBOSE} && echo "$0: test ${TEST}: wrong number of Recipients in ${SRVL}"
    ERR=true
  fi
else
  ${VERBOSE} && echo "$0: test ${TEST}: mcpmts.sh failed"
  ERR=true
fi
if ${ERR}
then
  ERRS=`expr ${ERRS} + 1 `
  echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
fi

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  rm -f ${QMGRCONFMAP} ${MTMAP} ${OUT}
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
