/*
 * Copyright (c) 2014 Claus Assmann
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the license/LICENSE.3C file which can be found at the
 * top level of this source code distribution.
 *
 *	$Id: tlscertpinning.h,v 1.6 2014/01/28 03:30:05 ca Exp $
 */

#ifndef SM_TLSCERTPINNING_H
#define SM_TLSCERTPINNING_H 1
#include "sm/generic.h"
#if MTA_USE_TLS && FFR_CERT_PINNING
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/bio.h>
#include <openssl/pem.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>

/* TLS cert pinning context: subset of tlsi_ctx; just to pass data around */
struct tlscp_ctx_S
{
	int			 tlscp_vrfy;
	int			 tlscp_hs_stat;
	uint32_t	 tlscp_hs_tricks;
	time_T		 tlscp_notafter_t;
	uint		 tlscp_cert_mdlen;
	uchar		 tlscp_cert_md[EVP_MAX_MD_SIZE];
};

typedef struct tlscp_ctx_S	tlscp_ctx_T, *tlscp_ctx_P;

void cpcertfpinfo(tlscp_ctx_P _dst, tlscp_ctx_P _src);
void clrcertfpinfo(tlscp_ctx_P _tlscp);

/*
 * should this be a bit field (flags)
 * or can it be just a counter to go through various options?
 * the current flags cannot be combined...
 * depends on what we are trying as workarounds...
 */
#define THS_TLS1_0	0x00000001
#define THS_TLS1_1	0x00000002
#define THS_TLS1_2	0x00000003

/* :.s/SSL_\(OP_.*\)/#ifdef &
#define TLS_\1 &
#else
#define TLS_\1 0
#endif/ */
#ifdef SSL_OP_NO_SSLv2
#define TLS_OP_NO_SSLv2 SSL_OP_NO_SSLv2
#else
#define TLS_OP_NO_SSLv2 0
#endif
#ifdef SSL_OP_NO_SSLv3
#define TLS_OP_NO_SSLv3 SSL_OP_NO_SSLv3
#else
#define TLS_OP_NO_SSLv3 0
#endif
#ifdef SSL_OP_NO_TLSv1
#define TLS_OP_NO_TLSv1 SSL_OP_NO_TLSv1
#else
#define TLS_OP_NO_TLSv1 0
#endif
#ifdef SSL_OP_NO_TLSv1_1
#define TLS_OP_NO_TLSv1_1 SSL_OP_NO_TLSv1_1
#else
#define TLS_OP_NO_TLSv1_1 0
#endif
#ifdef SSL_OP_NO_TLSv1_2
#define TLS_OP_NO_TLSv1_2 SSL_OP_NO_TLSv1_2
#else
#define TLS_OP_NO_TLSv1_2 0
#endif

#define TLSOPTS_TLS1_0	(TLS_OP_NO_SSLv2|TLS_OP_NO_SSLv3|TLS_OP_NO_TLSv1_1|TLS_OP_NO_TLSv1_2)
#define TLSOPTS_TLS1_1	(TLS_OP_NO_SSLv2|TLS_OP_NO_SSLv3|TLS_OP_NO_TLSv1|TLS_OP_NO_TLSv1_2)
#define TLSOPTS_TLS1_2	(TLS_OP_NO_SSLv2|TLS_OP_NO_SSLv3|TLS_OP_NO_TLSv1|TLS_OP_NO_TLSv1_1)

#endif
#endif /* SM_TLSCERTPINNING_H */
