/*
 * Copyright (c) 2011 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: confchk.c,v 1.3 2011/01/18 21:04:21 ca Exp $")
#include "sm/error.h"
#include "sm/sm-conf.h"
#include "sm/units.h"

#if MTA_USE_DKIM
#include "sm/dkimdef.h"
#include "dkim.h"

/*
**  DKIM_HDRTLENCB -- Callback for configuration: 
**
**	Parameters:
**		hdl -- conf handle
**		cbd -- callback context (NULL)
**		def -- current conf. definition
**		vd -- pointer to actual configuration data
**
**	Returns:
**		0: ok, errno otherwise
*/

int
dkim_hdrtlencb(sm_conf_T *hdl, void *cbd, sm_conf_definition_T const *def, void const *vd)
{
	uint32_t v;

	if (NULL == vd)
		return 0;
	v = *(uint32_t *)vd;
	if (v > 1 UNIT_GBYTES)
		return EDOM;
	if (v > 0 && v <= DKIM_HDRLEN)
		return EDOM;
	return 0;
}

/*
**  DKIM_HDRVLENCB -- Callback for configuration: 
**
**	Parameters:
**		hdl -- conf handle
**		cbd -- callback context (NULL)
**		def -- current conf. definition
**		vd -- pointer to actual configuration data
**
**	Returns:
**		0: ok, errno otherwise
*/

int
dkim_hdrvlencb(sm_conf_T *hdl, void *cbd, sm_conf_definition_T const *def, void const *vd)
{
	uint32_t v;

	if (NULL == vd)
		return 0;
	v = *(uint32_t *)vd;
	if (v > 1 UNIT_MBYTES)
		return EDOM;
	if (v > 0 && v <= DKIM_HDRLEN)
		return EDOM;
	return 0;
}
#endif /* MTA_USE_DKIM */
