/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_ipaddr2a.c,v 1.5 2014/03/01 17:07:18 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/ctype.h"
#include "sm/net.h"
#include "sm/str.h"
#if MTA_NETINET6
#include "sm/memops.h"
#endif

/*
**  SM_IPADDR2STR -- convert an IP address struct to an ASCII string
**
**	Parameters:
**		ipaddr -- address struct
**		dst -- append ASCII representation of address (output)
**
**	Returns:
**		error: ENOSPC
*/

sm_ret_T
sm_ipaddr2str(sm_ipaddr_P ipaddr, sm_str_P dst)
{
	SM_REQUIRE(ipaddr != NULL);

	switch (ipaddr->ipa_family) {
	  case AF_INET:
		return sm_inet_ipv4str(ipaddr->ipa_addr.ipv4_addr, dst);
		break;

#if MTA_NETINET6
	  case AF_INET6:
		{
		sm_sockaddr_T sockaddr;

		SCK_ASSIGN_IP(sockaddr, *ipaddr);
		return sm_sockaddr2str(&sockaddr, dst);
		}
		break;
#endif
	  default:
		return sm_error_perm(SM_EM_NONE, EPROTONOSUPPORT);
	}

	/* NOTREACHED */
	return sm_error_perm(SM_EM_NONE, EINVAL);
}
